@@ -22,6 +22,7 @@ import           Control.Applicative                   ((<$>), (<*>))
 import           Control.Concurrent                    (forkIO, threadDelay)
 import           Control.Concurrent.MVar               (MVar, newEmptyMVar,
                                                         takeMVar, tryPutMVar)
+import           Control.Concurrent.Async              (race_)
 import qualified Control.Exception                     as Ex
 import           Control.Monad                         (forever, unless, void,
                                                         when, forM)
@@ -66,42 +67,41 @@ import           GhcBuild                              (buildPackage,
 import qualified Config                                as GHC
 import           Data.Streaming.Network                (bindPortTCP)
 import           Network                               (withSocketsDo)
-#if MIN_VERSION_http_conduit(2, 0, 0)
 import           Network.HTTP.Conduit                  (conduitManagerSettings, newManager)
 import           Data.Default.Class                    (def)
-#else
-import           Network.HTTP.Conduit                  (def, newManager)
+#if MIN_VERSION_http_client(0,4,7)
+import           Network.HTTP.Client                   (managerSetProxy, noProxy)
 #endif
 import           Network.HTTP.ReverseProxy             (ProxyDest (ProxyDest),
                                                         waiProxyToSettings, wpsTimeout, wpsOnExc)
-#if MIN_VERSION_http_reverse_proxy(0, 2, 0)
 import qualified Network.HTTP.ReverseProxy             as ReverseProxy
-#endif
 import           Network.HTTP.Types                    (status200, status503)
 import           Network.Socket                        (sClose)
 import           Network.Wai                           (responseLBS, requestHeaders)
 import           Network.Wai.Parse                     (parseHttpAccept)
-import           Network.Wai.Handler.Warp              (run)
+import           Network.Wai.Handler.Warp              (run, defaultSettings, setPort)
+import           Network.Wai.Handler.WarpTLS           (runTLS, tlsSettingsMemory)
 import           SrcLoc                                (Located)
 import           Data.FileEmbed        (embedFile)
 
-lockFile :: DevelOpts -> FilePath
-lockFile _opts =  "yesod-devel/devel-terminate"
+lockFile :: FilePath
+lockFile =  "yesod-devel/devel-terminate"
 
 writeLock :: DevelOpts -> IO ()
 writeLock opts = do
     createDirectoryIfMissing True "yesod-devel"
-    writeFile (lockFile opts) ""
+    writeFile lockFile ""
     createDirectoryIfMissing True "dist" -- for compatibility with old devel.hs
     writeFile "dist/devel-terminate" ""
 
 removeLock :: DevelOpts -> IO ()
 removeLock opts = do
-    removeFileIfExists (lockFile opts)
+    removeFileIfExists lockFile
     removeFileIfExists "dist/devel-terminate"  -- for compatibility with old devel.hs
 
 data DevelTermOpt = TerminateOnEnter | TerminateOnlyInterrupt
      deriving (Show, Eq)
+     
 data DevelOpts = DevelOpts
       { isCabalDev   :: Bool
       , forceCabal   :: Bool
@@ -111,6 +111,7 @@ data DevelOpts = DevelOpts
       , failHook     :: Maybe String
       , buildDir     :: Maybe String
       , develPort    :: Int
+      , develTlsPort :: Int
       , proxyTimeout :: Int
       , useReverseProxy :: Bool
       , terminateWith :: DevelTermOpt
@@ -120,20 +121,34 @@ getBuildDir :: DevelOpts -> String
 getBuildDir opts = fromMaybe "dist" (buildDir opts)
 
 defaultDevelOpts :: DevelOpts
-defaultDevelOpts = DevelOpts False False False (-1) Nothing Nothing Nothing 3000 10 True TerminateOnEnter
+defaultDevelOpts = DevelOpts
+    { isCabalDev   = False
+    , forceCabal   = False
+    , verbose      = False
+    , eventTimeout = -1
+    , successHook  = Nothing
+    , failHook     = Nothing
+    , buildDir     = Nothing
+    , develPort    = 3000
+    , develTlsPort = 3443
+    , proxyTimeout = 10
+    , useReverseProxy = True
+    , terminateWith = TerminateOnEnter
+    }
 
 cabalProgram :: DevelOpts -> FilePath
-cabalProgram opts | isCabalDev opts = "cabal-dev"
-                  | otherwise       = "cabal"
+cabalProgram opts
+  | isCabalDev opts = "cabal-dev"
+  | otherwise       = "cabal"
 
 -- | Run a reverse proxy from port 3000 to 3001. If there is no response on
 -- 3001, give an appropriate message to the user.
 reverseProxy :: DevelOpts -> I.IORef Int -> IO ()
 reverseProxy opts iappPort = do
-#if MIN_VERSION_http_conduit(2, 0, 0)
-    manager <- newManager conduitManagerSettings
+#if MIN_VERSION_http_client(0,4,7)
+    manager <- newManager $ managerSetProxy noProxy conduitManagerSettings
 #else
-    manager <- newManager def
+    manager <- newManager conduitManagerSettings
 #endif
     let refreshHtml = LB.fromChunks $ return $(embedFile "refreshing.html")
     let onExc _ req
@@ -149,36 +164,40 @@ reverseProxy opts iappPort = do
                 ]
                 refreshHtml
 
-    let runProxy =
-            run (develPort opts) $ waiProxyToSettings
+    let proxyApp = waiProxyToSettings
                 (const $ do
                     appPort <- liftIO $ I.readIORef iappPort
                     return $
-#if MIN_VERSION_http_reverse_proxy(0, 2, 0)
                         ReverseProxy.WPRProxyDest
-#else
-                        Right
-#endif
                         $ ProxyDest "127.0.0.1" appPort)
                 def
-#if MIN_VERSION_wai(3, 0, 0)
                     { wpsOnExc = \e req f -> onExc e req >>= f
-#else
-                    { wpsOnExc = onExc
-#endif
                     , wpsTimeout =
                         if proxyTimeout opts == 0
                             then Nothing
                             else Just (1000000 * proxyTimeout opts)
                     }
                 manager
-    loop runProxy `Ex.onException` exitFailure
+        runProxyTls port app = do
+          let cert = $(embedFile "certificate.pem")
+              key = $(embedFile "key.pem")
+              tlsSettings = tlsSettingsMemory cert key
+          runTLS tlsSettings (setPort port defaultSettings) app
+        httpProxy = run (develPort opts) proxyApp
+        httpsProxy = runProxyTls (develTlsPort opts) proxyApp
+    putStrLn "Application can be accessed at:\n"
+    putStrLn $ "http://127.0.0.1:" ++ show (develPort opts)
+    putStrLn $ "https://127.0.0.1:" ++ show (develTlsPort opts)
+    putStrLn $ "If you wish to test https capabilities, you should set the following variable:"
+    putStrLn $ "  export APPROOT=https://127.0.0.1:" ++ show (develTlsPort opts)
+    putStrLn ""
+    loop (race_ httpProxy httpsProxy) `Ex.onException` exitFailure
   where
-    loop proxy = forever $ do
-        void proxy
-        putStrLn "Reverse proxy stopped, but it shouldn't"
+    loop proxies = forever $ do
+        void proxies
+        putStrLn $ "Reverse proxy stopped, but it shouldn't"
         threadDelay 1000000
-        putStrLn "Restarting reverse proxy"
+        putStrLn $ "Restarting reverse proxies"
 
 checkPort :: Int -> IO Bool
 checkPort p = do
@@ -190,7 +209,8 @@ checkPort p = do
             return True
 
 getPort :: DevelOpts -> Int -> IO Int
-getPort opts _ | not (useReverseProxy opts) = return $ develPort opts
+getPort opts _ 
+  | not (useReverseProxy opts) = return $ develPort opts
 getPort _ p0 =
     loop p0
   where
@@ -206,7 +226,7 @@ devel opts passThroughArgs = withSocketsDo $ withManager $ \manager -> do
     unlessM (checkPort $ develPort opts) $ error "devel port unavailable"
     iappPort <- getPort opts 17834 >>= I.newIORef
     when (useReverseProxy opts) $ void $ forkIO $ reverseProxy opts iappPort
-    checkDevelFile
+    develHsPath <- checkDevelFile
     writeLock opts
 
     let (terminator, after) = case terminateWith opts of
@@ -221,7 +241,7 @@ devel opts passThroughArgs = withSocketsDo $ withManager $ \manager -> do
       filesModified <- newEmptyMVar
       void $ forkIO $
         void $ watchTree manager "." (const True) (\_ -> void (tryPutMVar filesModified ()))
-      evalStateT (mainOuterLoop iappPort filesModified) Map.empty
+      evalStateT (mainOuterLoop develHsPath iappPort filesModified) Map.empty
     after
     writeLock opts
     exitSuccess
@@ -229,9 +249,13 @@ devel opts passThroughArgs = withSocketsDo $ withManager $ \manager -> do
     bd = getBuildDir opts
 
     -- outer loop re-reads the cabal file
-    mainOuterLoop iappPort filesModified = do
+    mainOuterLoop develHsPath iappPort filesModified = do
       ghcVer <- liftIO ghcVersion
+#if MIN_VERSION_Cabal(1,20,0)
+      cabal  <- liftIO $ D.tryFindPackageDesc "."
+#else
       cabal  <- liftIO $ D.findPackageDesc "."
+#endif
       gpd    <- liftIO $ D.readPackageDescription D.normal cabal
       ldar   <- liftIO lookupLdAr
       (hsSourceDirs, _) <- liftIO $ checkCabalFile gpd
@@ -245,20 +269,20 @@ devel opts passThroughArgs = withSocketsDo $ withManager $ \manager -> do
                                                , "yesod-devel/ldargs.txt"
                                                ]
              rebuild <- liftIO $ mkRebuild ghcVer cabal opts ldar
-             mainInnerLoop iappPort hsSourceDirs filesModified cabal rebuild
+             mainInnerLoop develHsPath iappPort hsSourceDirs filesModified cabal rebuild
            else do
              liftIO (threadDelay 5000000)
-             mainOuterLoop iappPort filesModified
+             mainOuterLoop develHsPath iappPort filesModified
 
     -- inner loop rebuilds after files change
-    mainInnerLoop iappPort hsSourceDirs filesModified cabal rebuild = go
+    mainInnerLoop develHsPath iappPort hsSourceDirs filesModified cabal rebuild = go
        where
          go = do
            _ <- recompDeps hsSourceDirs
            list <- liftIO $ getFileList hsSourceDirs [cabal]
            success <- liftIO rebuild
            pkgArgs <- liftIO (ghcPackageArgs opts)
-           let devArgs = pkgArgs ++ ["devel.hs"]
+           let devArgs = pkgArgs ++ [develHsPath]
            let loop list0 = do
                    (haskellFileChanged, list1) <- liftIO $
                        watchForChanges filesModified hsSourceDirs [cabal] list0 (eventTimeout opts)
@@ -300,7 +324,7 @@ devel opts passThroughArgs = withSocketsDo $ withManager $ \manager -> do
                    liftIO $ Ex.throwTo watchTid (userError "process finished")
            loop list
            n <- liftIO $ cabal `isNewerThan` (bd </> "setup-config")
-           if n then mainOuterLoop iappPort filesModified else go
+           if n then mainOuterLoop develHsPath iappPort filesModified else go
 
 runBuildHook :: Maybe String -> IO ()
 runBuildHook (Just s) = do
@@ -318,6 +342,8 @@ configure opts extraArgs =
   checkExit =<< createProcess (proc (cabalProgram opts) $
                                  [ "configure"
                                  , "-flibrary-only"
+                                 , "--disable-tests"
+                                 , "--disable-benchmarks"
                                  , "-fdevel"
                                  , "--disable-library-profiling"
                                  , "--with-ld=yesod-ld-wrapper"
@@ -337,7 +363,7 @@ removeFileIfExists file = removeFile file `Ex.catch` handler
 mkRebuild :: String -> FilePath -> DevelOpts -> (FilePath, FilePath) -> IO (IO Bool)
 mkRebuild ghcVer cabalFile opts (ldPath, arPath)
   | GHC.cProjectVersion /= ghcVer =
-       failWith "Yesod has been compiled with a different GHC version, please reinstall"
+       failWith "Yesod has been compiled with a different GHC version, please reinstall yesod-bin"
   | forceCabal opts               = return (rebuildCabal opts)
   | otherwise                     =
       return $ do
@@ -397,10 +423,18 @@ watchForChanges filesModified hsSourceDirs extraFiles list t = do
 
     isHaskell filename _ = takeExtension filename `elem` [".hs", ".lhs", ".hsc", ".cabal"]
 
-checkDevelFile :: IO ()
-checkDevelFile = do
-  e <- doesFileExist "devel.hs"
-  unless e $ failWith "file devel.hs not found"
+checkDevelFile :: IO FilePath
+checkDevelFile =
+    loop paths
+  where
+    paths = ["app/devel.hs", "devel.hs", "src/devel.hs"]
+
+    loop [] = failWith $ "file devel.hs not found, checked: " ++ show paths
+    loop (x:xs) = do
+        e <- doesFileExist x
+        if e
+            then return x
+            else loop xs
 
 checkCabalFile :: D.GenericPackageDescription -> IO ([FilePath], D.Library)
 checkCabalFile gpd = case D.condLibrary gpd of
@@ -473,7 +507,11 @@ lookupLdAr = do
 
 lookupLdAr' :: IO (Maybe (FilePath, FilePath))
 lookupLdAr' = do
+#if MIN_VERSION_Cabal(1,22,0)
+  (_, _, pgmc) <- D.configCompilerEx (Just D.GHC) Nothing Nothing D.defaultProgramConfiguration D.silent
+#else
   (_, pgmc) <- D.configCompiler (Just D.GHC) Nothing Nothing D.defaultProgramConfiguration D.silent
+#endif
   pgmc' <- D.configureAllKnownPrograms D.silent pgmc
   return $ (,) <$> look D.ldProgram pgmc' <*> look D.arProgram pgmc'
      where
