@@ -9,7 +9,7 @@ import qualified Data.Text as T
 import System.Exit
 import System.Process
 import Control.Monad
-import System.Directory
+import System.Directory hiding (findFiles)
 import Data.Maybe (mapMaybe)
 import qualified Filesystem.Path.CurrentOS as F
 import qualified Filesystem as F
@@ -17,6 +17,8 @@ import qualified Codec.Archive.Tar as Tar
 import Control.Exception
 import qualified Data.ByteString.Lazy as L
 import Codec.Compression.GZip (compress)
+import qualified Data.Foldable as Fold
+import Control.Monad.Trans.Writer (tell, execWriter)
 
 run :: String -> [String] -> IO ()
 run a b = do
@@ -25,8 +27,9 @@ run a b = do
 
 keter :: String -- ^ cabal command
       -> Bool -- ^ no build?
+      -> Bool -- ^ no copy to?
       -> IO ()
-keter cabal noBuild = do
+keter cabal noBuild noCopyTo = do
     ketercfg <- keterConfig
     mvalue <- decodeFile ketercfg
     value <-
@@ -36,7 +39,12 @@ keter cabal noBuild = do
                 case Map.lookup "host" value of
                     Just (String s) | "<<" `T.isPrefixOf` s ->
                         error $ "Please set your hostname in " ++ ketercfg
-                    _ -> return value
+                    _ ->
+                        case Map.lookup "user-edited" value of
+                            Just (Bool False) ->
+                                error $ "Please edit your Keter config file at "
+                                     ++ ketercfg
+                            _ -> return value
             Just _ -> error $ ketercfg ++ " is not an object"
 
     files <- getDirectoryContents "."
@@ -46,10 +54,18 @@ keter cabal noBuild = do
             [] -> error "No cabal file found"
             _ -> error "Too many cabal files found"
 
-    exec <-
-        case Map.lookup "exec" value of
-            Just (String s) -> return $ F.collapse $ "config" F.</> F.fromText s
-            _ -> error $ "exec not found in " ++ ketercfg
+    let findFiles (Object v) =
+            mapM_ go $ Map.toList v
+          where
+            go ("exec", String s) = tellFile s
+            go ("extraFiles", Array a) = Fold.mapM_ tellExtra a
+            go (_, v') = findFiles v'
+            tellFile s = tell [F.collapse $ "config" F.</> F.fromText s]
+            tellExtra (String s) = tellFile s
+            tellExtra _          = error "extraFiles should be a flat array"
+        findFiles (Array v) = Fold.mapM_ findFiles v
+        findFiles _ = return ()
+        bundleFiles = execWriter $ findFiles $ Object value
 
     unless noBuild $ do
         run cabal ["clean"]
@@ -58,15 +74,27 @@ keter cabal noBuild = do
 
     _ <- try' $ F.removeTree "static/tmp"
 
-    archive <- Tar.pack "" [F.encodeString exec, "config", "static"]
+    archive <- Tar.pack "" $
+        "config" : "static" : map F.encodeString bundleFiles
     let fp = T.unpack project ++ ".keter"
     L.writeFile fp $ compress $ Tar.write archive
 
-    case Map.lookup "copy-to" value of
+    unless noCopyTo $ case Map.lookup "copy-to" value of
         Just (String s) ->
-            case parseMaybe (.: "copy-to-port") value of
-                Just i -> run "scp" ["-P" ++ show (i :: Int), fp, T.unpack s]
-                Nothing -> run "scp" [fp, T.unpack s]
+            let baseArgs = [fp, T.unpack s] :: [String]
+
+                scpArgs =
+                    case parseMaybe (.: "copy-to-args") value of
+                        Just as -> as ++ baseArgs
+                        Nothing -> baseArgs
+
+                args =
+                    case parseMaybe (.: "copy-to-port") value of
+                        Just i -> "-P" : show (i :: Int) : scpArgs
+                        Nothing -> scpArgs
+
+            in run "scp" args
+
         _ -> return ()
   where
     -- Test for alternative config file extension (yaml or yml).
