@@ -15,12 +15,6 @@ import           Options                (injectDefaults)
 import qualified Paths_yesod_bin
 import           Scaffolding.Scaffolder
 
-#if MIN_VERSION_optparse_applicative(0,6,0)
-import           Options.Applicative.Types (ReadM (ReadM))
-#else
-import           Options.Applicative.Builder.Internal (Mod, OptionFields)
-#endif
-
 import           HsFile                 (mkHsFile)
 #ifndef WINDOWS
 import           Build                  (touch)
@@ -60,13 +54,21 @@ data Command = Init { _initBare :: Bool }
                      , develIgnore       :: [String]
                      , develExtraArgs    :: [String]
                      , _develPort        :: Int
+                     , _develTlsPort     :: Int
                      , _proxyTimeout     :: Int
                      , _noReverseProxy   :: Bool
                      , _interruptOnly    :: Bool
                      }
              | Test
              | AddHandler
-             | Keter { _keterNoRebuild :: Bool }
+                    { addHandlerRoute   :: Maybe String
+                    , addHandlerPattern :: Maybe String
+                    , addHandlerMethods :: [String]
+                    }
+             | Keter
+                    { _keterNoRebuild :: Bool
+                    , _keterNoCopyTo  :: Bool
+                    }
              | Version
   deriving (Show, Eq)
 
@@ -102,17 +104,25 @@ main = do
     Configure       -> cabal ["configure"]
     Build es        -> touch' >> cabal ("build":es)
     Touch           -> touch'
-    Keter noRebuild -> keter (cabalCommand o) noRebuild
+    Keter{..}       -> keter (cabalCommand o) _keterNoRebuild _keterNoCopyTo
     Version         -> putStrLn ("yesod-bin version: " ++ showVersion Paths_yesod_bin.version)
-    AddHandler      -> addHandler
+    AddHandler{..}  -> addHandler addHandlerRoute addHandlerPattern addHandlerMethods
     Test            -> cabalTest cabal
-    Devel{..}       -> devel (DevelOpts
-                              (optCabalPgm o == CabalDev) _develDisableApi (optVerbose o)
-                              _develRescan _develSuccessHook _develFailHook
-                              _develBuildDir _develPort _proxyTimeout
-                              (not _noReverseProxy)
-                              (if _interruptOnly then TerminateOnlyInterrupt else TerminateOnEnter )
-                           ) develExtraArgs
+    Devel{..}       -> let develOpts = DevelOpts
+                             { isCabalDev   = optCabalPgm o == CabalDev
+                             , forceCabal   = _develDisableApi
+                             , verbose      = optVerbose o
+                             , eventTimeout = _develRescan
+                             , successHook  = _develSuccessHook
+                             , failHook     = _develFailHook
+                             , buildDir     = _develBuildDir
+                             , develPort    = _develPort
+                             , develTlsPort = _develTlsPort
+                             , proxyTimeout = _proxyTimeout
+                             , useReverseProxy = not _noReverseProxy
+                             , terminateWith = if _interruptOnly then TerminateOnlyInterrupt else TerminateOnEnter
+                             }
+                       in devel develOpts develExtraArgs
   where
     cabalTest cabal = do touch'
                          _ <- cabal ["configure", "--enable-tests", "-flibrary-only"]
@@ -141,8 +151,9 @@ optParser = Options
                             (progDesc "Run project with the devel server"))
                       <> command "test"      (info (pure Test)
                             (progDesc "Build and run the integration tests"))
-                      <> command "add-handler" (info (pure AddHandler)
-                            (progDesc "Add a new handler and module to the project"))
+                      <> command "add-handler" (info addHandlerOptions
+                            (progDesc ("Add a new handler and module to the project."
+                            ++ " Interactively asks for input if you do not specify arguments.")))
                       <> command "keter"       (info keterOptions
                             (progDesc "Build a keter bundle"))
                       <> command "version"     (info (pure Version)
@@ -150,17 +161,13 @@ optParser = Options
                       )
 
 keterOptions :: Parser Command
-keterOptions = Keter <$> switch ( long "nobuild" <> short 'n' <> help "Skip rebuilding" )
+keterOptions = Keter
+    <$> switch ( long "nobuild" <> short 'n' <> help "Skip rebuilding" )
+    <*> switch ( long "nocopyto" <> help "Ignore copy-to directive in keter config file" )
 
 defaultRescan :: Int
 defaultRescan = 10
 
-#if MIN_VERSION_optparse_applicative(0,10,0)
-option' = option auto
-#else
-option' = option
-#endif
-
 develOptions :: Parser Command
 develOptions = Devel <$> switch ( long "disable-api"  <> short 'd'
                             <> help "Disable fast GHC API rebuilding")
@@ -168,7 +175,7 @@ develOptions = Devel <$> switch ( long "disable-api"  <> short 'd'
                             <> help "Run COMMAND after rebuild succeeds")
                      <*> optStr ( long "failure-hook" <> short 'f' <> metavar "COMMAND"
                             <> help "Run COMMAND when rebuild fails")
-                     <*> option' ( long "event-timeout" <> short 't' <> value defaultRescan <> metavar "N"
+                     <*> option auto ( long "event-timeout" <> short 't' <> value defaultRescan <> metavar "N"
                             <> help ("Force rescan of files every N seconds (default "
                                      ++ show defaultRescan
                                      ++ ", use -1 to rely on FSNotify alone)") )
@@ -178,9 +185,11 @@ develOptions = Devel <$> switch ( long "disable-api"  <> short 'd'
                                    <> help "ignore file changes in DIR" )
                               )
                      <*> extraCabalArgs
-                     <*> option' ( long "port" <> short 'p' <> value 3000 <> metavar "N"
+                     <*> option auto ( long "port" <> short 'p' <> value 3000 <> metavar "N"
                             <> help "Devel server listening port" )
-                     <*> option' ( long "proxy-timeout" <> short 'x' <> value 0 <> metavar "N"
+                     <*> option auto ( long "tls-port" <> short 'q' <> value 3443 <> metavar "N"
+                            <> help "Devel server listening port (tls)" )
+                     <*> option auto ( long "proxy-timeout" <> short 'x' <> value 0 <> metavar "N"
                             <> help "Devel server timeout before returning 'not ready' message (in seconds, 0 for none)" )
                      <*> switch ( long "disable-reverse-proxy" <> short 'n'
                             <> help "Disable reverse proxy" )
@@ -192,20 +201,19 @@ extraCabalArgs = many (strOption ( long "extra-cabal-arg" <> short 'e' <> metava
                                    <> help "pass extra argument ARG to cabal")
                       )
 
+addHandlerOptions :: Parser Command
+addHandlerOptions = AddHandler
+    <$> optStr ( long "route" <> short 'r' <> metavar "ROUTE"
+           <> help "Name of route (without trailing R). Required.")
+    <*> optStr ( long "pattern" <> short 'p' <> metavar "PATTERN"
+           <> help "Route pattern (ex: /entry/#EntryId). Defaults to \"\".")
+    <*> many (strOption ( long "method" <> short 'm' <> metavar "METHOD"
+                 <> help "Takes one method. Use this multiple times to add multiple methods. Defaults to none.")
+             )
+
 -- | Optional @String@ argument
 optStr :: Mod OptionFields (Maybe String) -> Parser (Maybe String)
-optStr m =
-#if MIN_VERSION_optparse_applicative(0,10,0)
-    nullOption (success . str) $ value Nothing <> m
-#else
-    nullOption $ value Nothing <> reader (success . str)  <> m
-#endif
-  where
-#if MIN_VERSION_optparse_applicative(0,6,0)
-    success = ReadM . Right
-#else
-    success = Right
-#endif
+optStr m = option (Just <$> str) $ value Nothing <> m
 
 -- | Like @rawSystem@, but exits if it receives a non-success result.
 rawSystem' :: String -> [String] -> IO ()
